// Arduino base scale light controller for RC applications
// Written by Scott White, drzoo2@yahoo.com, January 03 2015
// Revision 001 Modified from Scale Light Controller project Revision 005 for Wraith Project. Includes looking at Channel 3 throttle position
              //to perform other light functions.
// Revision 002 Released. Added police, hazard and off functions to be controlled via channel3
// Revision 003 Added ability to calibrate endpoints for where scale blinkers turn on. Now writing to two more memory locations.
              // Fixed a bug in reverse steering that caused the brake lights to flash when kINTstp = 0
// Revison 004 Modified for Arduino NANO. Update Pin-outs. Added a 4th channel to control Winch. Added a 5th channel to turn LED Light bar ON/OFF. Added 6th channel to control lightbar brightness.
// Revision 005 Fixed timing issues with lights. Now uses a timer so functions can change immediately.
            // Lots of bug fixes. Only loop now is Winch. This should become new base code.
            
// User adjustable Assignments----------------------------------------------------------
const int kINTstp = 1;       // 0 = Just brakelights, no blinky. 1 = Declares Stoplights into Blinky. Pin setups stay the same
const int kFlash = 250;      // Blinker Speed in (mS).
const int kHazz = 300;       // Hazzard lED blink speed. 
const int kFlasher = 25;     // Calibration led flash speed
const int kBRTblk = 70;      // Blinker Running Brightness. Only use small changes. Default 50
const int kBRThl = 20;       // Headlight Running Brightness. Only use small changes. Default 25
const int kBRTbrk = 70;      // Brakelight Running Brightness. Only use small changes. Default 30
unsigned long kBRTlb = 0;    // LED Lightbar Running Brightness. Make this adjustable. Default Value
const int kPulseDur = 30000; // Duration to look for receiver pulse. Each channel is 2000, so 6 channel  12000. Used minimize any delays
//----------------------------------------------------------------------------------------

// Do not touch these! Unless you know what you are doing
const int kRSVCH5 = 4;       //Lightbar Brightness control
const int kRSVCH4 = 2;       //Winch Control
const int kRSVCH3 = 1;       // LED Light Function Control
const int kRSVCH2 = 12;      // Setup Throttle pin port
const int kRSVCH1 = 13;      // Setup Steering pin port
const int kRRLED = 9;        // Right Rear Stop LED
const int kLRLED = 6;        // Left Rear Stop LED
const int kRFLED = 5;        // Righ Blinker LED
const int kLFLED = 3;        // Left Blinker LED
const int kHLLED = 10;       // Headlight LEDs
const int kBARLED = 11;      // LED Light Bar
const int kWCHout = 7;       // Winch OUT
const int kWCHin = 8;        // Winch IN

unsigned long iVal;          // General unsigned Variable
unsigned long iValCH1;       // Channel 1 READ Variable
unsigned long iValCH2;       // Channel 2 READ Variable
unsigned long iValCH3;       // Channel 3 READ Variable
unsigned long iValCH4;       // Channel 4 READ Variable
unsigned long iValCH5;       // Channel 5 READ Variable
unsigned long iValCALST;     // Cal Steering General Read Variable
unsigned long iValCALSTR;    // Calibrated Deadzone Channel 1 255
unsigned long iValCALSTL;    // Calibrated Deadzone Channel 1 0
unsigned long iValCALTHR;    // Cal Throttle General Read Variable
unsigned long iValCALFWD;    // Calibrated Deadzone Channel 2 255
unsigned long iValCALRVS;    // Calibrated Deadzone Channel 2 0

int kCali = 1;               // Calibrate on restart
int kPROMST = 0;             //EEPROM Variable for Steering position
int kPROMDRV = 0;            //EEPROM Variable for Driving position
int kPROMBLR = 0;            //EEPROM Variable for Steering Blink Time Right
int kPROMBLL = 0;            //EEPROM Variable for Steering Blink Time Left

long iNterval = 1000;        //millisecond interval
long iPrevTime = 0;          //Keeps track of previous value
int iBlink = 0;              //0 = LED off, 1 = LED ON
int iCount = 0;              //Set counter to known value
int iSpace = 0;              //Set Counter to a known value
int LEDstate = 0;            //Are LEDs on or off?

#include <EEPROM.h>

void setup (){
  // Pin Functions
  //pinMode(kRRLED, OUTPUT);  //NOW A PWM OUTPUT
  //pinMode(kLRLED, OUTPUT);  //NOW A PWM OUTPUT
  //pinMode(kRFLED, OUTPUT);  //NOW A PWM OUTPUT
  //pinMode(kLFLED, OUTPUT);  //NOW A PWM OUTPUT
  //pinMode(kHLLED, OUTPUT);  //NOW A PWM OUTPUT
  //pinMode(kBARLED, OUTPUT); //NOW A PWM OUTPUT
  pinMode(kWCHout, OUTPUT);  //PIN7
  pinMode(kWCHin, OUTPUT);  //PIN8
  pinMode(kRSVCH1, INPUT);  //PIN13
  pinMode(kRSVCH2, INPUT);  //PIN12
  pinMode(kRSVCH3, INPUT);  //Pin1
  pinMode(kRSVCH4, INPUT);  //Pin0
  pinMode(kRSVCH5, INPUT);  //Pin4
}


// Calibration Function
// This code can be called at power on by holding both throttle and steering, any position
void calibration(){    
  analogWrite(kRRLED, 0); //Set LED states to off
  analogWrite(kLRLED, 0);
  analogWrite(kRFLED, 0);
  analogWrite(kLFLED, 0);
  analogWrite(kHLLED, 0);
  analogWrite(kBARLED,0);
  digitalWrite(kWCHout, LOW);  //PIN7
  digitalWrite(kWCHin, LOW);  //PIN8
  delay (1000);   // Delay enshures user can change forward/reverse on remote after power on
  // Look for special input to change to reversed channel on remote
  // if value is around 1.5mS skip. Needs to be wide, close as possible to 1mS or 2mS
  //  else look to see which direction is triggered
  // store value in RAM and so appropriate function runs
  // EEPROM.write(address, value);
  
  iValCH1 = pulseIn(kRSVCH1, HIGH, kPulseDur);
  if (iValCH1 < 1250 || iValCH1 > 1750){    // Look to see if user is holding steering posistion for calibration mode
     //If TRUE, Start Channel Calibrations mode, else go center calibrations
     
    //This is Programming mode. Loop until completed
    //Check if user has released mode wait until sure
    iVal = 0;
    while(iVal == 0){    //Wait here until user releases remote
      iValCH1 = pulseIn(kRSVCH1, HIGH, kPulseDur); 
      analogWrite(kRRLED, 255);  //Flash all LEDS to incate in cal mode.
      analogWrite(kLRLED, 255);
      analogWrite(kRFLED, 255);
      analogWrite(kLFLED, 255);
      analogWrite(kHLLED, 255);
      delay(50);
      analogWrite(kRRLED, 0); 
      analogWrite(kLRLED, 0);
      analogWrite(kRFLED, 0);
      analogWrite(kLFLED, 0);
      analogWrite(kHLLED, 0);
      delay(50);
      
      if (iValCH1 > 1250 && iValCH1 < 1750){    // Look if user is still holding steering posistion for calibration mode. Continue when false
        iVal = 1;
        analogWrite(kRRLED, 0); 
        analogWrite(kLRLED, 0);
        analogWrite(kRFLED, 0);
        analogWrite(kLFLED, 0);
        analogWrite(kHLLED, 0);
        delay(500);
      } 
    }
    
    //Turn on Blinky to indicate Steering calibration
    analogWrite(kRFLED, 255);
    analogWrite(kLFLED, 255);
    iVal = 0;
    while(iVal == 0){  //Wait here until user makes a steering change      
      iValCH1 = pulseIn(kRSVCH1, HIGH, kPulseDur); // Look to see if user is holding steering posistion for channel setup
      if (iValCH1 > 1600){ //channel read forward
        EEPROM.write(0, 0); //steering address 0
        iVal = 1;
      }
      if (iValCH1 < 1400){ //channel read reversed
        EEPROM.write(0, 1); //steering address 0
        iVal = 1;
      }      
    }
    //Turn off Blinky to indicate steering complete
    analogWrite(kRFLED, 0);
    analogWrite(kLFLED, 0);
    delay(1000);
 
    //Turn on HL and Stop light to indicate driving calibration
    analogWrite(kRRLED, 255); 
    analogWrite(kLRLED, 255);
    analogWrite(kHLLED, 255);
    iVal = 0;
    while(iVal == 0){
      iValCH2 = pulseIn(kRSVCH2, HIGH, kPulseDur);  // Look to see if user is holding throttle posistion for channel setup
      if (iValCH2 < 1350){ //channel read forward
      EEPROM.write(1, 0);  //driving address 1
      iVal = 1;
      }
    if (iValCH2 > 1650){ //channel read reversed
      EEPROM.write(1, 1);  //driving address 1
      iVal = 1;
      }      
    }
    //Turn off HL/Stop lights to indicate throttle complete
    analogWrite(kHLLED, 0);
    analogWrite(kRRLED, 0);
    analogWrite(kLRLED, 0);
    
    // Boundry scan to make sure positions are close to center and user is not hold one or the other accidently. 
    iVal = 0; 
    while (iVal == 0){
      iValCH1 = pulseIn(kRSVCH1, HIGH, kPulseDur); 
      iValCH2 = pulseIn(kRSVCH2, HIGH, kPulseDur);  
      if (iValCH1 < 1400 || iValCH1 > 1600 || iValCH2 < 1400 || iValCH2 > 1600){    // Look to see if user is holding any posistion
        iVal = 0;
      }
      else{
        iVal = 1;
      }
    }
    delay(1000);
    
    // Calibrate Steering blinker "ON" percentage
    iVal = 0;
    while(iVal == 0){
      analogWrite(kRFLED, 255);
      iValCH2 = pulseIn(kRSVCH2, HIGH, kPulseDur);  // Look to see if user is holding throttle posistion for channel setup
      iValCH1 = pulseIn(kRSVCH1, HIGH, kPulseDur);
      if (iValCH2 < 1400 || iValCH2 > 1600){ //channel 1 read for indication
        iValCH1 = iValCH1 / 10;
        iValCH1 = round(iValCH1);
        EEPROM.write(2, iValCH1);  //LED RIGHT BLINK address 2
        iVal = 1;
        analogWrite(kRFLED, 0);
      }     
    }
    
    // Boundry scan to make sure positions are close to center and user is not hold one or the other accidently. 
    iVal = 0; 
    while (iVal == 0){
      iValCH1 = pulseIn(kRSVCH1, HIGH, kPulseDur); 
      iValCH2 = pulseIn(kRSVCH2, HIGH, kPulseDur);  
      if (iValCH1 < 1400 || iValCH1 > 1600 || iValCH2 < 1400 || iValCH2 > 1600){    // Look to see if user is holding any posistion
        iVal = 0;
      }
      else{
        iVal = 1;
      }
    }
    delay(1000);
  
    iVal = 0;
    while(iVal == 0){
      analogWrite(kLFLED, 255);
      iValCH2 = pulseIn(kRSVCH2, HIGH, kPulseDur);  // Look to see if user is holding throttle posistion for channel setup
      iValCH1 = pulseIn(kRSVCH1, HIGH, kPulseDur);
      if (iValCH2 < 1400 || iValCH2 > 1600){ //channel 1 read for indication
        iValCH1 = iValCH1 / 10;
        iValCH1 = round(iValCH1);
        EEPROM.write(3, iValCH1);  //LED RIGHT BLINK address 2
        iVal = 1;
        analogWrite(kLFLED, 0);
        }     
    }  
  }
  
  // Boundry scan to make sure positions are close to center and user is not hold one or the other accidently. 
  iVal = 0; 
  while (iVal == 0){
    iValCH1 = pulseIn(kRSVCH1, HIGH, kPulseDur); 
    iValCH2 = pulseIn(kRSVCH2, HIGH, kPulseDur);  
    if (iValCH1 < 1400 || iValCH1 > 1600 || iValCH2 < 1400 || iValCH2 > 1600){    // Look to see if user is holding any posistion
      iVal = 0;
    }
    else{
      iVal = 1;
    }
  }
  delay(1000);
  
  //This code always runs once at power on
  delay(500);
  iValCALST = 0;
  iValCALTHR = 0;
  for(int x = 0; x < 5; x++){  
    iValCH1 = pulseIn(kRSVCH1, HIGH, kPulseDur); // Read steering posistion
    iValCALST = iValCH1 + iValCALST;
      
    iValCH2 = pulseIn(kRSVCH2, HIGH, kPulseDur);  // Read throttle posistion
    iValCALTHR = iValCH2 + iValCALTHR;  
        
    analogWrite(kRRLED, 255);  // Pretty Flashing
    delay (kFlasher);
    analogWrite(kRRLED, 0);
    delay (kFlasher);    
    analogWrite(kLRLED, 255);
    delay (kFlasher);
    analogWrite(kLRLED, 0);
    delay (kFlasher);
    analogWrite(kLFLED, 255);
    delay (kFlasher);
    analogWrite(kRFLED, 0);
    delay (kFlasher);
    analogWrite(kHLLED, 255);
    delay (kFlasher);
    analogWrite(kHLLED, 0);
    delay (kFlasher);
    analogWrite(kRFLED, 255);
    delay (kFlasher);
    analogWrite(kLFLED, 0);
    delay (kFlasher);
  }
  
  // Look at flag in RAM to call appropriate funtion. Set RAM if any other value other than 0 or 1
  kPROMST = EEPROM.read(0);  //read steering address data
  kPROMDRV = EEPROM.read(1);  //read driving address data
  
  if (kPROMST == 0){          // If = 0, Channel is normal
  kPROMBLR = EEPROM.read(2);
  kPROMBLL = EEPROM.read(3);
  }
  if (kPROMST == 1){          // If = 1, Channel is reversed
  kPROMBLL = EEPROM.read(2);
  kPROMBLR = EEPROM.read(3);
  }
  kPROMBLR = kPROMBLR * 10;
  kPROMBLL = kPROMBLL * 10;
  
  iValCALST = iValCALST / 5;        // Take average steering posistion
  iValCALSTR = kPROMBLR;    // Set Deadzone HIGH
  iValCALSTL = kPROMBLL;     // Set Deadzone LOW
    
  iValCALTHR = iValCALTHR / 5;      // Take average throttle posistion
  iValCALFWD = iValCALTHR * 1.02;    // Set Deadzone LOW
  iValCALRVS = iValCALTHR * 0.99;    // Set Deadzone HIGH
 
  for(int x = 0; x < 10; x++){      // More Pretty Flashing
    analogWrite(kRRLED, 255);
    analogWrite(kLRLED, 255);
    analogWrite(kLFLED, 255);
    analogWrite(kRFLED, 255);
    analogWrite(kHLLED, 255);
    delay (50);
    analogWrite(kRFLED, 0);
    analogWrite(kLFLED, 0);
    analogWrite(kRRLED, 0);
    analogWrite(kLRLED, 0);
    analogWrite(kHLLED, 0);
    delay (50);
  }
}


void protection(){
  //Protection, Sense if transmitter is off at power on. Flash LEDs in loop until signal received.
  iValCH1 = pulseIn(kRSVCH1, HIGH, kPulseDur); // Watch reciever CH1 for changes.
  if (iValCH1 < 500){
    kCali = 1;
    iVal = 0;
    while (iVal == 0){
      digitalWrite(kWCHout, LOW);  //PIN7
      digitalWrite(kWCHin, LOW);  //PIN8
      analogWrite(kRRLED, 25);
      analogWrite(kLRLED, 25);
      analogWrite(kRFLED, 25);
      analogWrite(kLFLED, 25);
      analogWrite(kHLLED, 25);
      analogWrite(kBARLED, 0);
      delay(1000);
      analogWrite(kRRLED, 0);
      analogWrite(kLRLED, 0);
      analogWrite(kRFLED, 0);
      analogWrite(kLFLED, 0);
      analogWrite(kHLLED, 0);
      analogWrite(kBARLED, 0);
      delay(1000);
      iValCH1 = pulseIn(kRSVCH1, HIGH, kPulseDur); // Watch reciever CH1 for changes.
      if (iValCH1 > 500){
        iVal = 1;
      }
    }
  }
}


void hazzards(){
  long iNterval = kHazz;            //millisecond interval  
  unsigned long iCurrTime = millis(); //Get Current time
    
  if(iCurrTime - iPrevTime > iNterval){
    iPrevTime = iCurrTime;
    if(iBlink == 0){
      iBlink = 1;
    }
    else{
      iBlink = 0;
    }
  }
        
  if (iBlink == 1){
    analogWrite(kRRLED, 255);
    analogWrite(kLRLED, 255);
    analogWrite(kRFLED, 255);
    analogWrite(kLFLED, 255);
    analogWrite(kHLLED, kBRThl);
    analogWrite(kBARLED, kBRTlb);
  }
    
  if (iBlink == 0){
    analogWrite(kRRLED, 0);
    analogWrite(kLRLED, 0);
    analogWrite(kRFLED, 0);
    analogWrite(kLFLED, 0);
  }
}

void spacy(){
  long iNterval = kFlasher;            //millisecond interval  
  unsigned long iCurrTime = millis(); //Get Current time
    
  if(iCurrTime - iPrevTime > iNterval){
    iPrevTime = iCurrTime;
    if(iBlink == 0){
      iBlink = 1;
    }
    else{
      iBlink = 0;
    }
  }
  //REAR LED LIGHTS
  if(iSpace < 3){
    if(iCount == 0){
      if (iBlink == 1){
        analogWrite(kRRLED, 255);
        analogWrite(kLRLED, 255);
        iCount = 1;
      }   
    }
    if(iCount == 1){
      if (iBlink == 0){
        analogWrite(kRRLED, 0);
        analogWrite(kLRLED, 0);
        iCount = 0;
        iSpace ++;
      }
    }
  }
  //SIDE MARKER LED LIGHTS
  if(iSpace > 2 && iSpace < 6){
    if(iCount == 0){
      if (iBlink == 1){
        analogWrite(kRFLED, 255);
        analogWrite(kLFLED, 255);
        iCount = 1;
      }   
    }
    if(iCount == 1){
      if (iBlink == 0){
        analogWrite(kRFLED, 0);
        analogWrite(kLFLED, 0);
        iCount = 0;
        iSpace ++;
      }
    }
  }    
  //Head Light LED LIGHTS
  if(iSpace > 5 && iSpace < 9){
    if(iCount == 0){
      if (iBlink == 1){
        analogWrite(kHLLED, 255);
        iCount = 1;
      }   
    }
    if(iCount == 1){
      if (iBlink == 0){
        analogWrite(kHLLED, 0);
        iCount = 0;
        iSpace ++;
      }
    }
  }        
  //Light BAR LED LIGHTS
  if(iSpace > 7){
    if(iCount == 0){
      if (iBlink == 1){
        analogWrite(kBARLED, 255);
        iCount = 1;
      }   
    }
    if(iCount == 1){
      if (iBlink == 0){
        analogWrite(kBARLED, 0);
        iCount = 0;
        iSpace ++;
      }
    }
  }           
  if(iSpace >= 12){
    iSpace = 0;
  }  
}


// Steering Blinker Function
// When called
void signalsST(){
  //Steering normal channel 1
  long iNterval = kFlash;            //millisecond interval  
  unsigned long iCurrTime = millis(); //Get Current time
  if(iCurrTime - iPrevTime > iNterval){
    iPrevTime = iCurrTime;
    if(iBlink == 0){
    iBlink = 1;
    }
  else{
   iBlink = 0;
  }
  }
  
  if (kPROMST == 0){      
    if (iValCH1 > iValCALSTR){
      if(iBlink == 0){  
        analogWrite(kRFLED, 0);    
        if (kINTstp == 1){
          analogWrite(kRRLED, 0);
                    
          if (kPROMDRV == 0){
            if(iValCH2 < iValCALRVS){
              //do nothing
            }
            else{
              //analogWrite(kRRLED, kBRTbrk);
              analogWrite(kLRLED, kBRTbrk);
            }   
          }
          if (kPROMDRV == 1){
            if(iValCH2 > iValCALFWD){   
              //do nothing
            }
            else{
              //analogWrite(kRRLED, kBRTbrk);
              analogWrite(kLRLED, kBRTbrk);
            }               
          }    
        }
      }
      if(iBlink == 1){   
        analogWrite(kRFLED, 255);   
        if (kINTstp == 1){
          analogWrite(kRRLED, 255);
          
          if (kPROMDRV == 0){
            if(iValCH2 < iValCALRVS){
              //do nothing
            }
            else{
              //analogWrite(kRRLED, kBRTbrk);
              analogWrite(kLRLED, kBRTbrk);
            }   
          }
          if (kPROMDRV == 1){
            if(iValCH2 > iValCALFWD){   
              //do nothing
            }
            else{
              //analogWrite(kRRLED, kBRTbrk);
              analogWrite(kLRLED, kBRTbrk);
            }               
          }              
        }
      }
    }
    
    if (iValCH1 < iValCALSTL){
      if(iBlink == 0){
        analogWrite(kLFLED, 0);
        if (kINTstp == 1){      
          analogWrite(kLRLED, 0);
          
          if (kPROMDRV == 0){
            if(iValCH2 < iValCALRVS){
              //do nothing
            }
            else{
              analogWrite(kRRLED, kBRTbrk);
              //analogWrite(kLRLED, kBRTbrk);
            }   
          }
          if (kPROMDRV == 1){
            if(iValCH2 > iValCALFWD){   
              //do nothing
            }
            else{
              analogWrite(kRRLED, kBRTbrk);
              //analogWrite(kLRLED, kBRTbrk);
            }               
          }          
        }
        
      }
      if(iBlink == 1){
        analogWrite(kLFLED, 255);
        if (kINTstp == 1){      
          analogWrite(kLRLED, 255);
          
          if (kPROMDRV == 0){
            if(iValCH2 < iValCALRVS){
              //do nothing
            }
            else{
              analogWrite(kRRLED, kBRTbrk);
              //analogWrite(kLRLED, kBRTbrk);
            }   
          }
          if (kPROMDRV == 1){
            if(iValCH2 > iValCALFWD){   
              //do nothing
            }
            else{
              analogWrite(kRRLED, kBRTbrk);
              //analogWrite(kLRLED, kBRTbrk);
            }               
          }          
        }
      }
    }
  }
  
  //Steering reverse channel 1
  if (kPROMST == 1){
    if (iValCH1 < iValCALSTL){
      if(iBlink == 0){
        analogWrite(kRFLED, 0);
        if (kINTstp == 1){      
          analogWrite(kRRLED, 0);
          
          if (kPROMDRV == 0){
            if(iValCH2 < iValCALRVS){
              //do nothing
            }
            else{
              //analogWrite(kRRLED, kBRTbrk);
              analogWrite(kLRLED, kBRTbrk);
            }   
          }
          if (kPROMDRV == 1){
            if(iValCH2 > iValCALFWD){   
              //do nothing
            }
            else{
              //analogWrite(kRRLED, kBRTbrk);
              analogWrite(kLRLED, kBRTbrk);
            }               
          }              
        }
      }
      if(iBlink ==1){
        analogWrite(kRFLED, 255);
        if (kINTstp == 1){      
          analogWrite(kRRLED, 255);
          
          if (kPROMDRV == 0){
            if(iValCH2 < iValCALRVS){
              //do nothing
            }
            else{
              //analogWrite(kRRLED, kBRTbrk);
              analogWrite(kLRLED, kBRTbrk);
            }   
          }
          if (kPROMDRV == 1){
            if(iValCH2 > iValCALFWD){   
              //do nothing
            }
            else{
              //analogWrite(kRRLED, kBRTbrk);
              analogWrite(kLRLED, kBRTbrk);
            }               
          }          
          
          
        }
      }
    }
//--------------------------------------------------------------------------    
    if (iValCH1 > iValCALSTR){
      if(iBlink == 0){
        analogWrite(kLFLED, 0);
        if (kINTstp == 1){
          analogWrite(kLRLED, 0);
          if (kPROMDRV == 0){
            if(iValCH2 < iValCALRVS){
              //do nothing
            }
            else{
              analogWrite(kRRLED, kBRTbrk);
              //analogWrite(kLRLED, kBRTbrk);
            }   
          }
          if (kPROMDRV == 1){
            if(iValCH2 > iValCALFWD){   
              //do nothing
            }
            else{
              analogWrite(kRRLED, kBRTbrk);
              //analogWrite(kLRLED, kBRTbrk);
            }               
          }
                    
        }
      }
      if(iBlink == 1){
        analogWrite(kLFLED, 255); 
        if (kINTstp == 1){
          analogWrite(kLRLED, 255);
          
          if (kPROMDRV == 0){
            if(iValCH2 < iValCALRVS){
              //do nothing
            }
            else{
              analogWrite(kRRLED, kBRTbrk);
              //analogWrite(kLRLED, kBRTbrk);
            }   
          }
          if (kPROMDRV == 1){
            if(iValCH2 > iValCALFWD){   
              //do nothing
            }
            else{
              analogWrite(kRRLED, kBRTbrk);
              //analogWrite(kLRLED, kBRTbrk);
            }               
          }
          
          
          
          
        }
      }
    }
  }  
}


// Brakelight Function
void signalsTHR(){ 
  //Brake Lights normal channel 2
  if (kPROMDRV == 0){
    if(iValCH2 < iValCALRVS){ 
      //Steering 
      if (kPROMST == 0){      
        if (iValCH1 > iValCALSTR){
          //analogWrite(kRRLED, 255);
          analogWrite(kLRLED, 255);
        }
        else if (iValCH1 < iValCALSTL){
          analogWrite(kRRLED, 255);
          //analogWrite(kLRLED, 255);      
        }
        else{
          analogWrite(kRRLED, 255);
          analogWrite(kLRLED, 255);      
        }
      }
      
      if (kPROMST == 1){              
        if (iValCH1 < iValCALSTL){
        //analogWrite(kRRLED, 255);
        analogWrite(kLRLED, 255);        
        }  
        else if (iValCH1 > iValCALSTR){
        analogWrite(kRRLED, 255);
        //analogWrite(kLRLED, 255);        
        }
        else{
          analogWrite(kRRLED, 255);
          analogWrite(kLRLED, 255);      
        }        
      }
    }
    //Head Lights normal channel 2
    if(iValCH2 > iValCALFWD){     
      analogWrite(kHLLED, 255);
      if (iValCH1 > iValCALSTR || iValCH1 < iValCALSTL){
        //do nothing
      }
      else{
        analogWrite(kRRLED, kBRTbrk);
        analogWrite(kLRLED, kBRTbrk);          
      }
    }
  } 
  
    //Brake Lights reverse channel 2
  if (kPROMDRV == 1){
    if(iValCH2 > iValCALFWD){    
      //Steering 
      if (kPROMST == 0){      
        if (iValCH1 > iValCALSTR){
          //analogWrite(kRRLED, 255);
          analogWrite(kLRLED, 255);
        }
        else if (iValCH1 < iValCALSTL){
          analogWrite(kRRLED, 255);
          //analogWrite(kLRLED, 255);      
        }
        else{
          analogWrite(kRRLED, 255);
          analogWrite(kLRLED, 255);      
        }        
      }
      
      if (kPROMST == 1){              
        if (iValCH1 < iValCALSTL){
        //analogWrite(kRRLED, 255);
        analogWrite(kLRLED, 255);        
        }  
        else if (iValCH1 > iValCALSTR){
        analogWrite(kRRLED, 255);
        //analogWrite(kLRLED, 255);        
        }
        else{
          analogWrite(kRRLED, 255);
          analogWrite(kLRLED, 255);      
        }        
      }
    }
    //Head Lights reverse channel 2
    if(iValCH2 < iValCALRVS){     
      analogWrite(kHLLED, 255);
      if (iValCH1 > iValCALSTR || iValCH1 < iValCALSTL){
        //do nothing
      }
      else{
        analogWrite(kRRLED, kBRTbrk);
        analogWrite(kLRLED, kBRTbrk);          
      }      
    }
  } 
}


void loop (){
  protection();
  
  if (kCali == 1){  // Call calibration function, run only once on startup
    kCali = 0;
    calibration();
  }
    
  //This function turns LEDs OFF
  iValCH3 = pulseIn(kRSVCH3, HIGH, kPulseDur);
  if (iValCH3 < 1165){ //mid value is 1ms
    // Set Running LED Brightness
    LEDstate = 0;
    analogWrite(kRRLED, 0);
    analogWrite(kLRLED, 0);
    analogWrite(kRFLED, 0);
    analogWrite(kLFLED, 0);
    analogWrite(kHLLED, 0);
    analogWrite(kBARLED, 0);
  }

  
  //This function turns LEDs on for Scale operation.
  if (iValCH3 > 1165 && iValCH3 < 1495){ //mid value is 1.33ms
    LEDstate = 1;
    iValCH1 = pulseIn(kRSVCH1, HIGH, kPulseDur); // Watch reciever CH1 for changes.
    if (iValCH1 > iValCALSTR || iValCH1 < iValCALSTL){
      signalsST();
    }
    else{
      analogWrite(kRFLED, kBRTblk);
      analogWrite(kLFLED, kBRTblk);
    }    
    iValCH2 = pulseIn(kRSVCH2, HIGH, kPulseDur); // Watch reciever CH2 for changes.
    if (iValCH2 < iValCALRVS || iValCH2 > iValCALFWD){
      signalsTHR();
    }
    else{
      analogWrite(kHLLED, kBRThl);
      if(iValCH1 > iValCALSTR || iValCH1 < iValCALSTL){          
        //do nothing
      }
      else{
        analogWrite(kRRLED, kBRTbrk); 
        analogWrite(kLRLED, kBRTbrk);  
      }       
    }   
    analogWrite(kBARLED, kBRTlb);  
  }


  //This function turns LEDs on for Hazard operation.
  if (iValCH3 > 1495 && iValCH3 < 1825){ //mid value is 1.66ms
    LEDstate = 1;
    hazzards();
  }


  //This function turns LEDs on for Spacy operation.
  if (iValCH3 > 1825){ //mid value is 2ms
    LEDstate = 1;
    spacy();
  }
  
  
  // LED Lightbar Brightness control
  iValCH5 = pulseIn(kRSVCH5, HIGH, kPulseDur);
  if (iValCH5 < 1165){ //mid value is 1ms
    kBRTlb = 0;
  }
  if (iValCH5 > 1165 && iValCH5 < 1495){ //mid value is 1.33ms
    kBRTlb = 2; 
  }   
  if (iValCH5 > 1495 && iValCH5 < 1825){ //mid value is 1.66ms    
    kBRTlb = 25;
  }
  if (iValCH5 > 1825){ //mid value is 2ms
    kBRTlb = 255;
  }
  
  
  // Winch Control. For precise control, Enter do loop for each mode
  iValCH4 = pulseIn(kRSVCH4, HIGH, kPulseDur);
  if (iValCH4 < 1165 || iValCH4 > 1825){ //mid value is 1 or 2ms
    if (iValCH4 > 500 && iValCH4 < 1165){
      iVal = 0;
      while (iVal == 0){
        digitalWrite(kWCHout, HIGH);  //PIN7
        digitalWrite(kWCHin, LOW);  //PIN8
        if(LEDstate == 1){
          hazzards();
        }
        iValCH4 = pulseIn(kRSVCH4, HIGH, kPulseDur);
        if (iValCH4 > 1165){
          digitalWrite(kWCHout, LOW);  //PIN7
          digitalWrite(kWCHin, LOW);  //PIN8
          iVal = 1;
        }
      }
    }
    if (iValCH4 > 1825){
      iVal = 0;
      while (iVal == 0){
        digitalWrite(kWCHout, LOW);  //PIN7
        digitalWrite(kWCHin, HIGH);  //PIN8
        if(LEDstate == 1){
          hazzards();
        }
        iValCH4 = pulseIn(kRSVCH4, HIGH, kPulseDur);
        if (iValCH4 < 1825){
          digitalWrite(kWCHout, LOW);  //PIN7
          digitalWrite(kWCHin, LOW);  //PIN8
          iVal = 1; 
        }
      }
    }
  }
}
